﻿using CarRepairShopContracts.BusinessLogicsContracts;
using CarRepairShopContracts.SearchModels;
using CarRepairShopContracts.StoragesContracts;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CarRepairShopView
{
    public partial class FormSellRepairs : Form
    {
        private readonly IRepairLogic _rlogic;
        private readonly IShopLogic _slogic;
        private readonly ILogger _logger;
        public FormSellRepairs(ILogger<FormSellRepairs> logger, IRepairLogic repairLogic, IShopLogic shopLogic)
        {
            InitializeComponent();
            _rlogic = repairLogic;
            _slogic = shopLogic;
            _logger = logger;
        }

        private void FormSellRepairs_Load(object sender, EventArgs e)
        {
            try
            {
                var list = _rlogic.ReadList(null);
                if (list != null)
                {
                    comboBoxRepairs.DisplayMember = "RepairName";
                    comboBoxRepairs.ValueMember = "Id";
                    comboBoxRepairs.DataSource = list;
                    comboBoxRepairs.SelectedItem = null;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSell_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (comboBoxRepairs.SelectedValue == null)
            {
                MessageBox.Show("Выберите Ремонт", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Продажа");

            try
            {
                var operationResult = _slogic.SellRepairs(_rlogic.ReadElement(new RepairSearchModel()
                {
                    Id = Convert.ToInt32(comboBoxRepairs.SelectedValue)
                })!, Convert.ToInt32(textBoxCount.Text));

                if (!operationResult)
                {
                    throw new Exception("Ошибка при продаже. Дополнительная информация в логах.");
                }

                MessageBox.Show("Продажа прошла успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка продажи");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
